@@ -1,14 +1,11 @@
 package WWW::Mailchimp;
-use Moo;
+use Moose;
 use LWP;
 use JSON;
 use URI;
-use URI::Escape;
-use PHP::HTTPBuildQuery qw(http_build_query);
-use MooX::Types::MooseLike::Base qw(Int InstanceOf Num Str);
-use Sub::Name;
+use URI::QueryParam;
 
-our $VERSION = '0.009_01';
+our $VERSION = '0.001';
 $VERSION = eval $VERSION;
 
 =head1 NAME
@@ -17,34 +14,27 @@ WWW::Mailchimp - Perl wrapper around the Mailchimp v1.3 API
 
 =head1 SYNOPSIS
 
-  use strict;
-  use WWW::Mailchimp
+use strict;
+use WWW::Mailchimp
 
-  my $mailchimp = WWW::Mailchimp->new(apikey => $apikey);
-  # defaults ( datacenter => 'us1', timeout => 5, output_format => 'json', api_version => 1.3 )
+my $mailchimp = WWW::Mailchimp->new(apikey => $apikey); # defaults ( datacenter => 'us1', timeout => 5, output_format => 'json', api_version => 1.3 )
 
-  my $campaigns = $mailchimp->campaigns;
-  my $lists = $mailchimp->lists;
-  my $subscribers = $mailchimp->listMembers( $lists->{data}->[0]->{id} );
-  my $ok = $mailchimp->listSubscribe( id => $lists->{data}->[0]->{id},
-                                      email_address => 'foo@bar.com',
-                                      update_existing => 1,
-                                      merge_vars => [ FNAME => 'foo',
-                                                      LNAME => 'bar' ] );
+my $campaigns = $mailchimp->campaigns;
+my $lists = $mailchimp->lists;
+my $subscribers = $mailchimp->listMembers( $lists->[0]->{id} );
+my $ok = $mailchimp->listSubscribe( id => $lists->[0]->{id}, email_address => 'foo@bar.com', update_existing => 1, merge_vars => [ FNAME => 'foo', LNAME => 'bar' ] );
 
 =head1 DESCRIPTION
 
 WWW::Mailchimp is a simple Perl wrapper around the Mailchimp API v1.3.
-
+  
 It is as simple as creating a new WWW::Mailchimp object and calling ->method
 Each key/value pair becomes part of a query string, for example:
 
   $mailchimp->listSubscribe( id => 1, email_address => 'foo@bar.com' );
 
-results in the query string
-
-  ?method=listSubscribe&id=1&email_address=foo@bar.com
-  # apikey, output, etc are tacked on by default. This is also uri_escaped
+results in the query string: ?method=listSubscribe&id=1&email_address=foo@bar.com
+apikey, output, etc are tacked on by default. This is also uri_escaped
 
 =head1 BUGS
 
@@ -62,8 +52,6 @@ Mail::Chimp::API - Perl wrapper around the Mailchimp v1.2 API using XMLRPC
 
 Justin Hunter <justin.d.hunter@gmail.com>
 
-Fayland Lam
-
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2011 by Justin Hunter
@@ -75,75 +63,64 @@ the same terms as the Perl 5 programming language system itself.
 
 has api_version => (
   is => 'ro',
-  isa => Num,
+  isa => 'Num',
   lazy => 1,
-  default => sub { 1.3 },
+  default => 1.3,
 );
 
 has datacenter => (
-  is => 'rw',
-  isa => Str,
+  is => 'ro',
+  isa => 'Str',
   lazy => 1,
-  default => sub { 'us1' },
+  default => 'us1',
 );
 
 has apikey => (
   is => 'ro',
-  isa => Str,
+  isa => 'Str',
   required => 1,
-  trigger => sub {
-    my ($self, $val) = @_;
-    my ($datacenter) = ($val =~ /\-(\w+)$/);
-    $self->datacenter($datacenter)
-  },
 );
 
 has api_url => (
   is => 'rw',
-  isa => Str,
+  isa => 'Str',
   lazy => 1,
   default => sub { my $self = shift; return 'https://' . $self->datacenter . '.api.mailchimp.com/' . $self->api_version . '/'; },
 );
 
 has output_format => (
   is => 'rw',
-  isa => Str,
+  isa => 'Str',
   lazy => 1,
-  default => sub { 'json' },
+  default => 'json',
 );
 
 has ua => (
-  is => 'lazy',
-  isa => InstanceOf['LWP::UserAgent'],
+  is => 'ro',
+  isa => 'LWP::UserAgent',
+  lazy => 1,
+  builder => '_build_lwp',
   handles => [ qw(request) ],
 );
 
 has timeout => (
   is => 'rw',
-  isa => Int,
+  isa => 'Int',
   lazy => 1,
-  default => sub { 5 },
+  default => 5,
 );
 
-has json => (
-  is => 'ro',
-  isa => InstanceOf['JSON'],
-  is => 'lazy',
-);
-
-sub _build_ua {
+sub _build_lwp {
   my $self = shift;
-  my $ua = LWP::UserAgent->new( timeout => $self->timeout, agent => __PACKAGE__ . ' ' . $VERSION, ssl_opts => { verify_hostname => 0, SSL_verify_mode => 0x00 } );
+  my $ua = LWP::UserAgent->new( timeout => $self->timeout, agent => __PACKAGE__ . ' ' . $VERSION );
 }
 
-sub _build_json { JSON->new->allow_nonref }
-
 sub _build_query_args {
   my ($self, %args) = @_;
-  my %merge_vars = @{delete $args{merge_vars} || []};
+  my %merge_vars = @{delete $args{merge_vars}};
   for my $var (keys %merge_vars) {
     if (ref($merge_vars{$var}) eq 'ARRAY') {
-      my $count = 0;
+      my $count = 0; 
       for my $val (@{$merge_vars{$var}}) {
         $args{"merge_vars[$var][$count]"} = $val;
         $count++;
@@ -152,13 +129,10 @@ sub _build_query_args {
       $args{"merge_vars[$var]"} = $merge_vars{$var};
     }
   }
-
+  
   my $uri = URI->new( $self->api_url );
-  $args{apikey} = $self->apikey;
-  $args{output} = $self->output_format;
-  delete $args{$_} for qw(json ua);
-
-  return \%args;
+  $uri->query_form_hash( apikey => $self->apikey, output => $self->output_format, %args );
+  return $uri;
 }
 
 sub _request {
@@ -166,17 +140,10 @@ sub _request {
   my $method = shift;
   my %args = ref($_[0]) ? %{$_[0]} : @_;
 
-  # uri must include the method (even for a POST request)
-  my $uri = URI->new( $self->api_url );
-  $uri->query( http_build_query( { method => $method } ) );
+  my $uri = $self->_build_query_args(method => $method, %args);
 
-  # build a POST request with json-encoded arguments
-  my $post_args = $self->_build_query_args(%args);
-  my $req = HTTP::Request->new('POST', $uri);
-  $req->content( uri_escape_utf8( $self->json->encode($post_args) ) );
-
-  my $response = $self->request( $req );
-  return $response->is_success ? $self->json->decode($response->content) : $response->status_line;
+  my $response = $self->request( HTTP::Request->new( GET => $uri->canonical ) );
+  return $response->is_success ? from_json($response->content) : $response->status_line;
 }
 
 my @api_methods = qw(
@@ -276,9 +243,8 @@ my @api_methods = qw(
   templates
 );
 
-no strict 'refs';
 for my $method (@api_methods) {
-  *{$method} = subname $method => sub { shift->_request($method, @_) };
+  __PACKAGE__->meta->add_method( $method => sub { shift->_request($method, @_) } );
 }
 
 1;
